"use strict";

var _react = _interopRequireDefault(require("react"));

var _chai = require("chai");

var _enzyme = require("enzyme");

var _sinon = _interopRequireDefault(require("sinon"));

var _moment = _interopRequireDefault(require("moment"));

var _datepicker = _interopRequireDefault(require("../datepicker"));

var _timepicker = _interopRequireDefault(require("../timepicker"));

var _keyConstants = require("../../../utils/keyConstants");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const textFind = text => el => el.text() === text;

describe('Datepicker', () => {
  let clock;
  let onChangeStub;
  beforeEach(() => {
    onChangeStub = _sinon.default.stub();
    const now = 1516627620000; // 2018-01-22 14:27.00 UTC+1

    clock = _sinon.default.useFakeTimers({
      now
    });
  });
  afterEach(() => {
    clock.restore();
  });
  describe('Rendering', () => {
    it('should render with a table', () => {
      const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_datepicker.default, null));
      (0, _chai.expect)(wrapper.find('table')).to.exist();
    });
    it('should render the currently selected month above the day grid', () => {
      const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_datepicker.default, null));
      (0, _chai.expect)(wrapper.find('tr').first().find('th').at(1).text()).to.contain('Januari 2018');
    });
    it('should show the weekdays', () => {
      const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_datepicker.default, null));
      (0, _chai.expect)(wrapper.find('tr').at(1).text()).to.equal('MåTiOnToFrLöSö');
    });
  });
  it('should highlight the given date when passing the getSelectedProp prop', () => {
    const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_datepicker.default, {
      selectedDate: (0, _moment.default)().add(2, 'week')
    }));
    (0, _chai.expect)(wrapper.find('.kui-datepicker__day--selected').text()).to.equal('5');
  });
  it('should display the given time when passing the getSelectedProp prop', () => {
    const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_datepicker.default, {
      selectedDate: (0, _moment.default)().add(3, 'hour').subtract(13, 'minutes')
    }));
    (0, _chai.expect)(wrapper.find('input').prop('value')).to.contain('17:14');
  });
  it('should call the given onChange handler as the date changes', () => {
    const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_datepicker.default, {
      onChange: onChangeStub
    }));
    wrapper.find('.kui-datepicker__next').simulate('click');
    wrapper.find('.kui-datepicker__day').filterWhere(textFind('12')).simulate('click');
    const expectedMomentInstance = (0, _moment.default)().add(1, 'month').date(12).toString();
    const actualMomentInstance = onChangeStub.getCall(0).args[0].toString();
    (0, _chai.expect)(actualMomentInstance).to.equal(expectedMomentInstance);
  });
  it('should change date while pressing arrow buttons', () => {
    const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_datepicker.default, {
      onChange: onChangeStub
    }));
    wrapper.simulate('focus');
    wrapper.simulate('keydown', {
      keyCode: _keyConstants.RIGHT_ARROW
    });
    const expectedMomentInstance = (0, _moment.default)().add(1, 'day').toString();
    const actualMomentInstance = onChangeStub.getCall(0).args[0].toString();
    (0, _chai.expect)(expectedMomentInstance).to.equal(actualMomentInstance);
  });
  describe('Timepicker', () => {
    it('should default to "--:--" if no date is specified', () => {
      const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_datepicker.default, null));
      (0, _chai.expect)(wrapper.find('input').prop('value')).to.contain('--:--');
    });
    it('should render a timepicker depending on a prop', () => {
      (0, _chai.expect)((0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_datepicker.default, null)).find(_timepicker.default)).to.exist();
      (0, _chai.expect)((0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_datepicker.default, {
        withoutTimepicker: true
      })).find(_timepicker.default)).to.not.exist();
    });
    it('should produce a moment instance while changing time', () => {
      const timepicker = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_datepicker.default, {
        onChange: onChangeStub
      })).find(_timepicker.default);
      const newHour = '15';
      const newMinute = '27';
      timepicker.find('input').simulate('change', {
        target: {
          value: `${newHour}:${newMinute}`
        }
      });
      (0, _chai.expect)(onChangeStub.getCall(0).args[0].toString()).to.equal((0, _moment.default)().hour(newHour).minute(newMinute).toString());
    });
  });
});