"use strict";

var _react = _interopRequireDefault(require("react"));

var _chai = require("chai");

var _enzyme = require("enzyme");

var _sinon = _interopRequireDefault(require("sinon"));

var _button = _interopRequireDefault(require("../../button/button"));

var _keyConstants = require("./../../../utils/keyConstants");

var _dropdown = _interopRequireDefault(require("../dropdown"));

var _DropdownOption = _interopRequireDefault(require("../DropdownOption"));

var _popover = _interopRequireDefault(require("../../popover/popover"));

var _badge = _interopRequireDefault(require("../../badge/badge"));

var _icon = _interopRequireDefault(require("../../icon/icon"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const options = [{
  value: 1,
  label: 'One'
}, {
  value: 2,
  label: 'Two'
}, {
  value: 3,
  label: 'Three'
}];

const textFind = text => node => node.text() === text;

const getOption = (dropdown, label) => dropdown.find(_DropdownOption.default).filterWhere(textFind(label));

describe('Dropdown', () => {
  it('should render a Dropdown', () => {
    const dropdown = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_dropdown.default, {
      placeholder: "This is our placeholdertext"
    }));
    (0, _chai.expect)(dropdown).to.have.text('This is our placeholdertext');
    (0, _chai.expect)(dropdown).to.have.className('drop-down');
  });
  it('should have a set type', () => {
    const dropdown = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_dropdown.default, {
      type: "ghost"
    }));
    (0, _chai.expect)(dropdown.props().type).to.equal('ghost');
  });
  it('should use the supplied className', () => {
    const dropdown = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_dropdown.default, {
      className: "one__two--three"
    }));
    (0, _chai.expect)(dropdown).to.have.className('one__two--three');
  });
  it('should not show options when closed', () => {
    const dropdown = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_dropdown.default, {
      options: options
    }));
    (0, _chai.expect)(dropdown.find('ul')).not.to.be.present();
  });
  it('should show/hide options on click', () => {
    const dropdown = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_dropdown.default, {
      options: options
    }));
    dropdown.find(_button.default).simulate('click');
    (0, _chai.expect)(dropdown.find(_DropdownOption.default)).to.exist();
    dropdown.find(_button.default).simulate('click');
    (0, _chai.expect)(dropdown.find(_DropdownOption.default)).to.not.exist();
  });
  it('setting selectedValue prop should select the correct option', () => {
    const selected = options[1];
    const dropdown = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_dropdown.default, {
      options: options,
      selectedValue: selected.value
    }));
    (0, _chai.expect)(dropdown).to.have.text(selected.label);
    dropdown.find(_button.default).simulate('click');
    (0, _chai.expect)(dropdown.find(_DropdownOption.default).at(1).prop('isSelected')).to.equal(true);
  });
  it('should call the onchange callback with the selected value even if it is falsy', () => {
    const onChangeStub = _sinon.default.stub();

    const ownOptions = [{
      value: 1,
      label: 'One'
    }, {
      value: 0,
      label: 'Zero'
    }, {
      value: false,
      label: 'False'
    }, {
      value: '',
      label: 'Empty string'
    }];
    const dropdown = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_dropdown.default, {
      options: ownOptions,
      selectedValue: 0,
      onChange: onChangeStub
    }));
    (0, _chai.expect)(dropdown).to.have.text('Zero');
    dropdown.find(_button.default).simulate('click');
    getOption(dropdown, 'False').simulate('click');
    (0, _chai.expect)(onChangeStub).to.have.been.calledWith(false);
  });
  it('should open and select with arrow/enter key', () => {
    const onChange = _sinon.default.stub();

    const dropdown = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_dropdown.default, {
      options: options,
      onChange: onChange
    }));
    dropdown.simulate('focus');
    (0, _chai.expect)(dropdown.find(_DropdownOption.default)).not.to.be.present();
    dropdown.simulate('keydown', {
      key: 'ArrowDown',
      keyCode: _keyConstants.KEY_DOWN
    });
    (0, _chai.expect)(dropdown.find(_DropdownOption.default)).to.be.present();
    dropdown.simulate('keydown', {
      key: 'ArrowDown',
      keyCode: _keyConstants.KEY_DOWN
    });
    dropdown.simulate('keydown', {
      keyCode: _keyConstants.KEY_ENTER,
      key: 'Enter'
    });
    (0, _chai.expect)(onChange).to.have.been.calledOnce();
    (0, _chai.expect)(onChange).to.have.been.calledWith(options[0].value);
  });
  it('should open and select with space key', () => {
    const onChange = _sinon.default.stub();

    const dropdown = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_dropdown.default, {
      options: options,
      onChange: onChange
    }));
    dropdown.simulate('focus');
    (0, _chai.expect)(dropdown.find(_DropdownOption.default)).not.to.be.present();
    dropdown.simulate('keydown', {
      keyCode: _keyConstants.KEY_DOWN,
      key: 'ArrowDown'
    });
    dropdown.simulate('keydown', {
      keyCode: _keyConstants.KEY_DOWN,
      key: 'ArrowDown'
    });
    (0, _chai.expect)(dropdown.find(_DropdownOption.default)).to.be.present();
    dropdown.simulate('keydown', {
      keyCode: _keyConstants.KEY_SPACE,
      key: ' '
    });
    (0, _chai.expect)(onChange).to.have.been.calledOnce();
    (0, _chai.expect)(onChange).to.have.been.calledWith(options[0].value);
  });
  it('should highlight option with mouse', () => {
    const dropdown = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_dropdown.default, {
      options: options
    }));
    dropdown.find(_button.default).simulate('click');
    dropdown.find(_DropdownOption.default).first().simulate('mouseEnter');
    (0, _chai.expect)(dropdown.find(_DropdownOption.default).first().find('div').hasClass('drop-down-option--highlighted')).to.equal(true);
    dropdown.find(_DropdownOption.default).last().simulate('mouseEnter');
    (0, _chai.expect)(dropdown.find(_DropdownOption.default).last()).to.have.className('drop-down-option--highlighted');
    dropdown.simulate('mouseLeave');
    (0, _chai.expect)(dropdown.find(_DropdownOption.default).last()).not.to.have.className('drop-down-option--highlighted');
  });
  it('should highlight option with keyboard', () => {
    const dropdown = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_dropdown.default, {
      options: options
    }));
    dropdown.find(_button.default).simulate('click');
    dropdown.simulate('keydown', {
      keyCode: _keyConstants.KEY_UP,
      key: 'ArrowUp'
    });
    (0, _chai.expect)(dropdown.find(_DropdownOption.default).last()).to.have.className('drop-down-option--highlighted');
    dropdown.simulate('keydown', {
      keyCode: _keyConstants.KEY_DOWN,
      key: 'ArrowDown'
    });
    (0, _chai.expect)(dropdown.find(_DropdownOption.default).first()).to.have.className('drop-down-option--highlighted');
  });
  it('should close on esc', () => {
    const dropdown = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_dropdown.default, {
      options: options
    }));
    dropdown.simulate('click');
    dropdown.simulate('keydown', {
      keyCode: _keyConstants.KEY_ESCAPE
    });
    (0, _chai.expect)(dropdown.find('ul')).not.to.be.present();
  });
  it('should show options above', () => {
    const dropdown = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_dropdown.default, {
      options: options,
      showOptionsAbove: true
    }));
    dropdown.find(_button.default).simulate('click');
    (0, _chai.expect)(dropdown.find(_popover.default).prop('down')).to.equal(false);
  });
  it('should not crash if a non existent selectedValue is used', () => {
    const dropdown = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_dropdown.default, {
      options: options,
      selectedValue: "a-random-value"
    }));
    (0, _chai.expect)(dropdown).to.be.present();
  });
  describe('Multiselect', () => {
    it('should show multiple selected values ', () => {
      const dropdown = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_dropdown.default, {
        multiple: true,
        options: options,
        selectedValue: [1, 2]
      }));
      const badgeTexts = dropdown.find(_badge.default).map(b => b.text());
      (0, _chai.expect)(badgeTexts).to.have.members(['One', 'Two']);
    });
    it('should filter out the correct value when clicking the icon on one of the badges', () => {
      const onChangeStub = _sinon.default.stub();

      const dropdown = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_dropdown.default, {
        multiple: true,
        onChange: onChangeStub,
        options: options,
        selectedValue: [1, 2]
      }));
      dropdown.find(_badge.default).filterWhere(badge => badge.text() === 'One').find(_icon.default).simulate('click');
      (0, _chai.expect)(onChangeStub).to.have.been.calledWith([2]);
    });
  });
});