import PropTypes from 'prop-types';
import React, { Component } from 'react';
import classnames from 'classnames';

import {
  KEY_DOWN,
  KEY_UP,
  KEY_BACKSPACE,
  KEY_LEFT,
  KEY_RIGHT,
} from './../../utils/keyConstants';

import TextField from '../textField/textField';
import Badge from '../badge/badge';
import Tooltip from '../tooltip/tooltip';

function overridePaste(e) {
  e.stopPropagation();
  e.preventDefault();
  const pastedText = e.clipboardData.getData('text/plain');

  document.execCommand('insertText', false, pastedText);
}

class Autocomplete extends Component {
  constructor(props) {
    super(props);
    this.state = {
      activeIndex: 0,
      selectedItemIndex: -1,
      inputIndex: undefined,
      inputFieldHasFocus: false,
      showTooltip: null,
    };
  }

  onKeyDown = e => {
    switch (e.keyCode) {
      case !this.shouldShowSuggestions() && KEY_UP:
      case KEY_LEFT:
        this.moveInputIfEmpty(-1, e.shiftKey);
        break;
      case !this.shouldShowSuggestions() && KEY_DOWN:
      case KEY_RIGHT:
        this.moveInputIfEmpty(1, e.shiftKey);
        break;
      case KEY_UP:
        e.preventDefault();
        this.navigateInDirection(-1);
        break;
      case KEY_DOWN:
        e.preventDefault();
        this.navigateInDirection(1);
        break;
      case this.props.handleBackspace && KEY_BACKSPACE:
        this.props.handleBackspace(e);
        break;
      case KEY_BACKSPACE:
        this.onRemoveByBackspace();
        break;
      default:
        break;
    }
  };

  onEnter = e => {
    e.preventDefault();
    const suggestions = this.getSuggestions();
    const { activeIndex } = this.state;
    if (suggestions[activeIndex]) {
      this.onAddSuggestion(suggestions[activeIndex], activeIndex);
    }
  };

  onAddSuggestion = async (suggestion, suggestionIndex) => {
    const { allowMultiple, items } = this.props;
    const { inputIndex = items.length } = this.state;
    if (!this.shouldAllowInput()) {
      return;
    }
    try {
      if (suggestion.id) {
        await this.props.onAdd(suggestion.id, suggestionIndex, inputIndex);
      } else {
        await this.props.onCreate(suggestion.label, inputIndex);
      }
    } catch (e) {
      return;
    }
    this.textFieldRef.clear();
    this.setState(
      {
        activeIndex: 0,
        inputIndex: inputIndex + 1,
      },
      () => {
        if (this.singleBadgeRef) {
          this.singleBadgeRef.focus();
        } else if (allowMultiple && this.textFieldRef) {
          this.textFieldRef.focus();
        }
      }
    );
  };

  onInputBlur = e => {
    if (e.relatedTarget !== this.componentRef) {
      this.setState({
        activeIndex: 0,
        inputFieldHasFocus: false,
        inputIndex: undefined,
      });
      this.props.onBlur(e);
    }
  };

  onInputChange = e => {
    if (!this.shouldAllowInput()) {
      this.textFieldRef.clear();
      return;
    }

    const { textContent } = e.target;
    this.setState({ activeIndex: 0 });
    this.props.onInputChange(textContent);
  };

  onInputFocus = e => {
    this.setState({ inputFieldHasFocus: true });
    this.props.onFocus(e);
  };

  onRemove = index => {
    const { items } = this.props;
    const removedItem = items[index];
    this.props.onRemove(
      removedItem.id,
      index,
      items.slice(0, index).concat(items.slice(index + 1))
    );
  };

  onRemoveByBackspace = () => {
    const { value, items } = this.props;
    const { inputIndex = items.length, selectedItemIndex } = this.state;
    if (value.length) {
      return;
    }
    switch (inputIndex) {
      case 0:
        return;
      case selectedItemIndex + 1:
        this.onRemove(selectedItemIndex);
        this.setState({
          inputIndex: inputIndex === items.length ? undefined : inputIndex - 1,
        });
        return;
      default:
        this.setState({ selectedItemIndex: inputIndex - 1 });
    }
  };

  onRemoveItemByClick = index => () => {
    this.onRemove(index);
    this.setState({ inputIndex: undefined }, this.focus);
  };

  setState(...args) {
    super.setState({ selectedItemIndex: -1 });
    super.setState(...args);
  }

  getSuggestions() {
    const { suggestions, items, value: inputValue } = this.props;

    if (this.shouldRenderCreateOption(suggestions, items)) {
      return suggestions.concat([{ label: `${inputValue}` }]);
    }
    return suggestions;
  }

  setTextFieldRef = ref => {
    this.textFieldRef = ref;
  };

  setSingleBadgeRef = ref => {
    this.singleBadgeRef = ref;
  };

  removeSingleItemByBackspace = e => {
    const { items } = this.props;
    if (e.key === 'Backspace' && items.length === 1) {
      this.onRemove(0);
      this.setState(
        {
          inputIndex: 0,
        },
        this.focus
      );
    }
  };

  moveInputIfEmpty = (change, shiftKey) => {
    const { value, items } = this.props;
    if (!value) {
      const { inputIndex = items.length } = this.state;
      const newInputIndex = Math.min(
        items.length,
        Math.max(0, inputIndex + change)
      );
      this.setState(
        Object.assign(
          {
            inputIndex:
              newInputIndex === items.length ? undefined : newInputIndex,
          },
          change === -1 &&
            shiftKey && { selectedItemIndex: newInputIndex, inputIndex },
          change === 1 && shiftKey && { selectedItemIndex: inputIndex }
        )
      );
    }
  };

  navigateInDirection = (direction = 1) => {
    const suggestionsLength = this.getSuggestions().length;
    const step = Math.sign(direction);
    const steppedIndex = Math.min(
      suggestionsLength - 1,
      Math.max(this.state.activeIndex + step, 0)
    );

    this.setState(
      {
        activeIndex: steppedIndex,
      },
      () => {
        if (
          this.selectedSuggestionRef.current &&
          this.selectedSuggestionRef.current.scrollIntoViewIfNeeded
        ) {
          this.selectedSuggestionRef.current.scrollIntoViewIfNeeded(false);
        }
      }
    );
  };

  selectedSuggestionRef = React.createRef();

  focus = () => {
    this.textFieldRef.focus();
  };

  shouldRenderCreateOption(suggestions, items) {
    if (!this.props.onCreate) {
      return false;
    }
    const { value: inputValue } = this.props;
    const inputValueIsEmpty = !inputValue.length;
    const inputValueIsInSuggestions = suggestions.find(
      s => s.label.toLowerCase() === inputValue.toLowerCase()
    );
    const inputValueIsInItems = items.find(
      i => i.label.toLowerCase() === inputValue.toLowerCase()
    );

    return (
      !inputValueIsEmpty && !inputValueIsInSuggestions && !inputValueIsInItems
    );
  }

  shouldShowSuggestions() {
    return (
      this.state.inputFieldHasFocus &&
      this.shouldAllowInput() &&
      this.getSuggestions().length > 0
    );
  }

  shouldAllowInput() {
    const { items, allowMultiple } = this.props;
    return allowMultiple || items.length === 0;
  }

  moveInputToIndex = inputIndex => () => {
    this.setState({ inputIndex }, this.focus);
  };

  toggleTooltip = (id = null) => {
    this.setState({ showTooltip: id });
  };

  renderLabel = (label, tooltip, id) => {
    if (!tooltip) return label;
    const { showTooltip } = this.state;
    const { text, direction = 'down', zIndex = 999 } = tooltip;

    return (
      <React.Fragment>
        <span
          onMouseEnter={() => this.toggleTooltip(id)}
          onMouseLeave={this.toggleTooltip}
        >
          {label}
        </span>
        {showTooltip === id && (
          <Tooltip direction={direction} zIndex={zIndex}>
            {text}
          </Tooltip>
        )}
      </React.Fragment>
    );
  };

  renderInput = inputProps => {
    const { items, allowMultiple } = this.props;
    const { inputIndex = items.length, selectedItemIndex } = this.state;
    const badges = items.map((item, index) => {
      const { id, label, tooltip = null } = item;
      return (
        <Badge
          className={classnames({
            'badge--dark': index === selectedItemIndex,
            'badge--tooltip': !!tooltip,
          })}
          tabIndex={allowMultiple ? undefined : 0}
          innerRef={allowMultiple ? undefined : this.setSingleBadgeRef}
          key={id}
          icon="remove"
          onIconClick={
            inputProps.disabled ? undefined : this.onRemoveItemByClick(index)
          }
          rightAlignedIcon
        >
          {this.renderLabel(label, tooltip, id)}
        </Badge>
      );
    });

    const inputSpan = (
      <span
        key="input-key"
        contentEditable={!inputProps.disabled && this.shouldAllowInput()}
        spellCheck={false}
        onPaste={overridePaste}
        {...inputProps}
        onInput={inputProps.onChange}
        onKeyDown={this.onKeyDown}
        className={classnames(
          inputProps.className,
          'autocomplete__input-field'
        )}
      />
    );

    const getPlaceholderInput = index => (
      <span
        key={`input-${index}`}
        className={classnames('autocomplete__input-field-placeholder', {
          'autocomplete--disabled': inputProps.disabled,
        })}
        onClick={inputProps.disabled ? undefined : this.moveInputToIndex(index)}
      />
    );

    const positionOfInput = Math.min(badges.length, inputIndex);

    return (
      <div className="autocomplete__pill-wrapper">
        {badges
          .reduce(
            (children, badge, index) =>
              children.concat(
                index === positionOfInput
                  ? inputSpan
                  : getPlaceholderInput(index),
                badge
              ),
            []
          )
          .concat(
            positionOfInput === badges.length
              ? inputSpan
              : getPlaceholderInput(badges.length)
          )}
      </div>
    );
  };

  renderSuggestionContainer = (suggestion, index) => {
    const { activeIndex } = this.state;
    const { renderSuggestion, renderCreateSuggestion } = this.props;
    const isActive = index === activeIndex;

    return (
      <li
        key={suggestion.id || 'unknown-id'}
        onClick={() => this.onAddSuggestion(suggestion, index)}
        item={suggestion}
        index={index}
        ref={isActive ? this.selectedSuggestionRef : null}
        className={classnames('autocomplete__suggestion', {
          'autocomplete__suggestion--active': isActive,
        })}
        onMouseEnter={() => this.setState({ activeIndex: index })}
      >
        {suggestion.id
          ? renderSuggestion(suggestion.id, suggestion.label, suggestion, index)
          : renderCreateSuggestion(
              suggestion.id,
              suggestion.label,
              suggestion,
              index
            )}
      </li>
    );
  };

  render() {
    const {
      className,
      items,
      value,
      renderSuggestion,
      renderCreateSuggestion,
      allowMultiple,
      ...otherProps
    } = this.props;

    otherProps.preventFocus = !this.shouldAllowInput();

    return (
      <div
        className={classnames('autocomplete', className)}
        tabIndex="-1"
        ref={r => {
          this.componentRef = r;
        }}
        onKeyDown={
          items.length === 1 && !allowMultiple
            ? this.removeSingleItemByBackspace
            : undefined
        }
      >
        <TextField
          {...otherProps}
          onFocus={this.onInputFocus}
          onBlur={this.onInputBlur}
          onChange={this.onInputChange}
          onEnter={this.onEnter}
          value={value}
          ref={this.setTextFieldRef}
          staticLabel={Boolean(
            items.length > 0 || this.shouldShowSuggestions()
          )}
          renderInput={this.renderInput}
          className={classnames('autocomplete__input', {
            'autocomplete__input--hidden': !this.shouldAllowInput(),
          })}
        />
        {this.shouldShowSuggestions() ? (
          <ul className="autocomplete__suggestions">
            {this.getSuggestions().map(this.renderSuggestionContainer)}
          </ul>
        ) : null}
      </div>
    );
  }
}

Autocomplete.propTypes = {
  allowMultiple: PropTypes.bool,
  className: PropTypes.string,
  items: PropTypes.arrayOf(
    PropTypes.shape({
      label: PropTypes.node.isRequired,
      id: PropTypes.string.isRequired,
      tooltip: PropTypes.shape({
        text: PropTypes.string,
        direction: PropTypes.oneOf(['up', 'down', 'left', 'right']),
        zIndex: PropTypes.number,
      }),
    })
  ).isRequired,
  value: PropTypes.string.isRequired,
  onCreate: PropTypes.func,
  onInputChange: PropTypes.func.isRequired,
  onAdd: PropTypes.func.isRequired,
  onRemove: PropTypes.func.isRequired,
  onFocus: PropTypes.func,
  onBlur: PropTypes.func,
  renderSuggestion: PropTypes.func,
  handleBackspace: PropTypes.func,
  renderCreateSuggestion: PropTypes.func,
  suggestions: PropTypes.arrayOf(
    PropTypes.shape({
      label: PropTypes.node.isRequired,
      id: PropTypes.string.isRequired,
    })
  ).isRequired,
};

Autocomplete.defaultProps = {
  allowMultiple: true,
  className: '',
  onCreate: null,
  handleBackspace: null,
  onFocus: () => {},
  onBlur: () => {},
  renderSuggestion: (id, label) => label,
  renderCreateSuggestion: (id, label) => `${label} (skapa ny)`,
};

Autocomplete.propDescriptions = {
  allowMultiple:
    'Whether to allow addition of multiple pills (items) or not. If false, the input field will not be shown after adding a pill, but will appear again if it is deleted.',
  className:
    'Add additional class names. The component will always have the "autocomplete" class.',
  items:
    'An array representing the items. Each item in the array needs to have at least an "id" and a "label" property. Optional property "tooltip" takes an object with three properties: the required "text", and the optional "direction" and "zIndex"',
  onCreate:
    'Callback that is fired when user creates a new pill given the current input value. If left out, creation is not allowed.',
  onInputChange:
    'Callback that is fired when the input value changes. Expected to be used to trigger fetching of suggestions.',
  renderSuggestion:
    'A function that is used to render a suggestion. If not specified, a standard rendering and styling will be given.',
  renderCreateSuggestion:
    'A function that is used to render a suggestion that will be created. If not specified, a standard rendering and styling will be given.',
  suggestions:
    'An array representing the suggestions. Each item in the array needs to have at least an "id" and a "label" property.',
};

Autocomplete.overridePropTypes = {
  items: { type: 'array', isRequired: 'Yes' },
  suggestions: { type: 'array', isRequired: 'Yes' },
};

export default Autocomplete;
