import React from 'react';
import { expect } from 'chai';
import { mount } from 'enzyme';
import { stub } from 'sinon';

import Autocomplete from '../autocomplete';
import Icon from '../../icon/icon';
import Badge from '../../badge/badge';
import Tooltip from '../../tooltip/tooltip';
import Spinner from '../../spinner/spinner';
import { KEY_DOWN, KEY_UP, KEY_BACKSPACE } from './../../../utils/keyConstants';

const suggestions = [
  { label: 'One', id: '11' },
  { label: 'Two', id: '22' },
  { label: 'Three', id: '33' },
];

const defaultProps = {
  loading: false,
  suggestions: [],
  items: [],
  allowMultiple: true,
  onInputChange: () => {},
  onAdd: () => {},
  onRemove: () => {},
  onCreate: null,
};

function mountAutocomplete(props) {
  let autocomplete;
  const onInputChange = value => {
    autocomplete.setProps({ value });
  };
  autocomplete = mount(
    <Autocomplete {...props} onInputChange={onInputChange} value="" />
  );
  return autocomplete;
}

function inputFieldOf(wrapper) {
  return wrapper.find('.autocomplete__input-field');
}

describe('Autocomplete', () => {
  let props;
  beforeEach(() => {
    props = Object.assign({}, defaultProps);
    stub(props, 'onAdd');
    stub(props, 'onRemove');
    stub(props, 'onInputChange');
  });

  afterEach(() => {
    props.onAdd.restore();
    props.onRemove.restore();
    props.onInputChange.restore();
  });

  it('should display suggestions when given by prop', () => {
    const autocomplete = mountAutocomplete({ ...props, suggestions });
    inputFieldOf(autocomplete)
      .simulate('focus')
      .simulate('input', { target: { textContent: 'One' } });
    expect(autocomplete.find('li')).to.have.length(3);
  });

  it('should show a loading spinner when fetching suggestions', () => {
    const autocomplete = mountAutocomplete({
      ...props,
      loading: true,
      suggestions,
    });
    inputFieldOf(autocomplete)
      .simulate('focus')
      .simulate('input', { target: { textContent: 'Four' } });
    expect(autocomplete.find(Spinner)).to.be.present();
  });

  it('should make the first suggestion item to the "active" one', () => {
    props.suggestions = suggestions;
    const autocomplete = mountAutocomplete(props);
    inputFieldOf(autocomplete).simulate('focus');
    expect(autocomplete.find('li').first()).to.have.className(
      'autocomplete__suggestion--active'
    );
  });

  it('should allow navigation with arrow keys and change the active suggestion', () => {
    props.suggestions = suggestions;
    const autocomplete = mountAutocomplete(props);
    inputFieldOf(autocomplete)
      .simulate('focus')
      .simulate('input', { target: { textContent: '' } })
      .simulate('keydown', { keyCode: KEY_DOWN });
    expect(autocomplete.find('li').first()).to.not.have.className(
      'autocomplete__suggestion--active'
    );
    expect(autocomplete.find('li').at(1)).to.have.className(
      'autocomplete__suggestion--active'
    );
    inputFieldOf(autocomplete).simulate('keydown', { keyCode: KEY_UP });
    expect(autocomplete.find('li').first()).to.have.className(
      'autocomplete__suggestion--active'
    );
    expect(autocomplete.find('li').at(1)).to.not.have.className(
      'autocomplete__suggestion--active'
    );
  });

  it('should not "wrap around" at the top or bottom when navigating', () => {
    props.suggestions = suggestions.slice(0, 2);
    const autocomplete = mountAutocomplete(props);
    inputFieldOf(autocomplete)
      .simulate('focus')
      .simulate('keydown', { keyCode: KEY_DOWN })
      .simulate('keydown', { keyCode: KEY_DOWN });
    expect(autocomplete.find('li').first()).to.not.have.className(
      'autocomplete__suggestion--active'
    );
    expect(autocomplete.find('li').at(1)).to.have.className(
      'autocomplete__suggestion--active'
    );
    inputFieldOf(autocomplete)
      .simulate('keydown', { keyCode: KEY_UP })
      .simulate('keydown', { keyCode: KEY_UP });
    expect(autocomplete.find('li').first()).to.have.className(
      'autocomplete__suggestion--active'
    );
    expect(autocomplete.find('li').at(1)).to.not.have.className(
      'autocomplete__suggestion--active'
    );
  });

  it('should reset the active item to the first one on input change', () => {
    const autocomplete = mountAutocomplete({ ...props, suggestions });
    inputFieldOf(autocomplete)
      .simulate('focus')
      .simulate('keydown', { keyCode: KEY_DOWN });
    expect(autocomplete.find('li').first()).to.not.have.className(
      'autocomplete__suggestion--active'
    );
    expect(autocomplete.find('li').at(1)).to.have.className(
      'autocomplete__suggestion--active'
    );
    inputFieldOf(autocomplete).simulate('input', {
      target: { textContent: 'One' },
    });
    expect(autocomplete.find('li').first()).to.have.className(
      'autocomplete__suggestion--active'
    );
    expect(autocomplete.find('li').at(1)).to.not.have.className(
      'autocomplete__suggestion--active'
    );
  });

  it('should be able to select an item with the mouse', () => {
    const autocomplete = mountAutocomplete({ ...props, suggestions });
    inputFieldOf(autocomplete).simulate('focus');
    autocomplete
      .find('li')
      .first()
      .simulate('click');
    expect(props.onAdd).to.have.been.calledOnce();
    expect(props.onAdd).to.have.been.calledWith(suggestions[0].id);
  });

  it('should be able to select an item with the enter key', () => {
    const autocomplete = mountAutocomplete({ ...props, suggestions });
    inputFieldOf(autocomplete)
      .simulate('focus')
      .simulate('keypress', { key: 'Enter' });
    expect(props.onAdd).to.have.been.calledOnce();
    expect(props.onAdd).to.have.been.calledWith(suggestions[0].id);
  });

  it('should not call onAdd when there are no suggestions', () => {
    const autocomplete = mountAutocomplete(props);
    inputFieldOf(autocomplete)
      .simulate('focus')
      .simulate('keypress', { key: 'Enter' });
    expect(props.onAdd).to.not.have.been.called();
  });

  it('should not display the input field after choosing one item if only one can be selected', () => {
    const autocomplete = mountAutocomplete({
      ...props,
      allowMultiple: false,
      items: [suggestions[0]],
      suggestions,
    });
    inputFieldOf(autocomplete).simulate('focus');
    expect(autocomplete.find('.autocomplete__input--hidden')).to.be.present();
  });

  it('should render pills for every selected item', () => {
    const autocomplete = mountAutocomplete({ ...props, items: suggestions });
    expect(autocomplete.find(Badge)).to.have.length(3);
  });

  it('should render pill tooltip for every selected item, if they are present', () => {
    const autocomplete = mountAutocomplete({
      ...props,
      allowMultiple: true,
      suggestions,
      items: [
        {
          id: 't3s7yMcTester',
          label: 'Klara Klarasson',
          tooltip: { test: 'klara@klarasson.se' },
        },
      ],
    });

    autocomplete
      .find(Badge)
      .find('span')
      .simulate('mouseenter');
    expect(autocomplete.find(Tooltip)).to.have.length(1);
  });

  it('should be possible to remove a selected item by clicking its icon', () => {
    const autocomplete = mountAutocomplete({
      ...props,
      suggestions,
      items: suggestions,
    });

    autocomplete
      .find(Icon)
      .first()
      .simulate('click');

    expect(props.onRemove).to.have.been.calledOnce();
    expect(props.onRemove).to.have.been.calledWith(suggestions[0].id);
  });

  it('should be possible to remove the item using backspace; first item is selected, then it is removed', () => {
    props.suggestions = suggestions;
    props.items = suggestions;
    const autocomplete = mountAutocomplete(props);
    inputFieldOf(autocomplete).simulate('keydown', { keyCode: KEY_BACKSPACE });
    expect(autocomplete.find(Badge).find('.badge--dark')).to.be.present();

    inputFieldOf(autocomplete).simulate('keydown', { keyCode: KEY_BACKSPACE });
    expect(props.onRemove).to.have.been.calledOnce();
    expect(props.onRemove).to.have.been.calledWith(suggestions[2].id);
  });

  it('should be able to remove the pill with backspace even if the text field is not there but is hidden', () => {
    props.suggestions = suggestions;
    props.items = suggestions;
    const autocomplete = mountAutocomplete(props);
    inputFieldOf(autocomplete).simulate('keydown', { keyCode: KEY_BACKSPACE });
    inputFieldOf(autocomplete).simulate('keydown', { keyCode: KEY_BACKSPACE });
    expect(props.onRemove).to.have.been.calledOnce();
    expect(props.onRemove).to.have.been.calledWith(suggestions[2].id);
  });

  it('should not remove any item if the input field has a value', () => {
    props.suggestions = suggestions;
    props.items = suggestions.slice(0, 2);
    const autocomplete = mountAutocomplete(props);
    inputFieldOf(autocomplete)
      .simulate('input', { target: { textContent: 'Two' } })
      .simulate('keydown', { keyCode: KEY_BACKSPACE });
    expect(autocomplete.find(Badge).find('.badge--dark')).not.to.be.present();
  });

  it('should show the option to create a new item if specified by prop', () => {
    props.onCreate = stub(props, 'onCreate');
    const autocomplete = mountAutocomplete(props);
    inputFieldOf(autocomplete)
      .simulate('focus')
      .simulate('input', {
        target: { textContent: 'Four' },
      });
    autocomplete.setProps({ suggestions });
    expect(
      autocomplete
        .find('li')
        .last()
        .text()
    ).to.include('skapa ny');
    expect(
      autocomplete
        .find('li')
        .last()
        .text()
    ).to.include('Four');
    props.onCreate.restore();
  });

  it('should show the option to create a new item even if there are no suggestions', () => {
    props.onCreate = stub(props, 'onCreate');
    const autocomplete = mountAutocomplete(props);
    inputFieldOf(autocomplete)
      .simulate('focus')
      .simulate('input', {
        target: { textContent: 'Anton' },
      });
    autocomplete.setProps({ suggestions: [] });
    expect(
      autocomplete
        .find('li')
        .last()
        .text()
    ).to.include('skapa ny');
    expect(
      autocomplete
        .find('li')
        .last()
        .text()
    ).to.include('Anton');
    props.onCreate.restore();
  });

  it('should call the prop for creating a new item when clicked', () => {
    props.onCreate = stub(props, 'onCreate');
    const autocomplete = mountAutocomplete(props);
    inputFieldOf(autocomplete)
      .simulate('focus')
      .simulate('input', {
        target: { textContent: 'Four' },
      });
    autocomplete.setProps({ suggestions });
    autocomplete
      .find('li')
      .last()
      .simulate('click');
    expect(props.onCreate).to.have.been.calledOnce();
    expect(props.onCreate).to.have.been.calledWith('Four');
    props.onCreate.restore();
  });

  it('should not show the option to create a new item if it is already a suggestion', () => {
    props.onCreate = stub(props, 'onCreate');
    const autocomplete = mountAutocomplete(props);
    inputFieldOf(autocomplete)
      .simulate('focus')
      .simulate('input', {
        target: { textContent: 'three' },
      });
    autocomplete.setProps({ suggestions });
    expect(
      autocomplete
        .find('li')
        .last()
        .text()
    ).to.not.include('skapa ny');
    props.onCreate.restore();
  });

  it('should not show the option to create a new item if it is already an item', () => {
    props.onCreate = stub(props, 'onCreate');
    props.items = [{ id: '10', label: 'Four' }];
    const autocomplete = mountAutocomplete(props);
    inputFieldOf(autocomplete)
      .simulate('focus')
      .simulate('input', {
        target: { textContent: 'Four' },
      });
    autocomplete.setProps({ suggestions });
    expect(
      autocomplete
        .find('li')
        .last()
        .text()
    ).to.not.include('skapa ny');
    props.onCreate.restore();
  });

  it('should not show the option to create a new item if input field is empty', () => {
    const onCreate = stub(props, 'onCreate');
    const autocomplete = mountAutocomplete({ ...props, onCreate });
    inputFieldOf(autocomplete)
      .simulate('focus')
      .simulate('input', { target: { textContent: '' } });
    autocomplete.setProps({ suggestions });
    expect(
      autocomplete
        .find('li')
        .last()
        .text()
    ).to.not.include('skapa ny');
    onCreate.restore();
  });

  it('should use a renderProp to render list items', () => {
    props.renderSuggestion = (id, label) => (
      <div>
        <h2>{id}</h2>
        <a>{label}</a>
      </div>
    );
    props.suggestions = suggestions;
    const autocomplete = mountAutocomplete(props);
    inputFieldOf(autocomplete).simulate('focus');
    expect(autocomplete.find('h2')).to.have.length(suggestions.length);
    expect(autocomplete.find('a')).to.have.length(suggestions.length);
  });

  it('should use a renderProp to render create-suggestions', () => {
    props.renderCreateSuggestion = (id, label) => (
      <div>`{label} (skapa mig)`</div>
    );
    props.onCreate = stub(props, 'onCreate');
    props.suggestions = [{ id: '1', label: 'gooo' }];
    const autocomplete = mountAutocomplete(props);
    inputFieldOf(autocomplete)
      .simulate('focus')
      .simulate('input', { target: { textContent: 'new' } });
    expect(
      autocomplete
        .find('li')
        .last()
        .text()
    ).to.include('new (skapa mig)');
    props.onCreate.restore();
  });
  it('should handle failed onAdd/onCreate', () => {
    const autocomplete = mountAutocomplete({
      ...props,
      onCreate: async () => {
        autocomplete.setProps({ errorText: 'error' });
        throw new Error('Was not added');
      },
    });
    inputFieldOf(autocomplete)
      .simulate('focus')
      .simulate('input', { target: { textContent: 'Hoppas det här fungerar' } })
      .simulate('keypress', { key: 'Enter' });
    expect(autocomplete.text()).to.include('Hoppas det här fungerar');
    expect(autocomplete.prop('errorText')).to.equal('error');
  });
});
