import PropTypes from 'prop-types';
import React, { Component } from 'react';
import classNames from 'classnames';

import Spinner from '../spinner/spinner';
import Icon from '../icon/icon';
import Tooltip from '../tooltip/tooltip';

function resolveClasses({
  active,
  compact,
  children,
  disabled,
  icon,
  loading,
  primary,
  roundLeft,
  roundRight,
  size,
  tooltip,
  type,
  rightAlignedIcon,
}) {
  const primaryClasses = {
    'button--standard': !primary,
    'button--primary': primary,
  };

  const iconClasses = {
    'button--icon': icon || loading,
    'button--icon--left': !rightAlignedIcon && children && (icon || loading),
    'button--icon--right': rightAlignedIcon && children && (icon || loading),
    'button--loading': loading,
  };

  const roundClasses = {
    'button--no-round-left': !roundLeft,
    'button--no-round-right': !roundRight,
  };

  const typeClasses = {
    'button--default': type === 'default' && !disabled,
    'button--success': type === 'success' && !disabled,
    'button--warning': type === 'warning' && !disabled,
    'button--danger': type === 'danger' && !disabled,
    'button--inverted': type === 'inverted' && !disabled,
    'button--ghost': type === 'ghost',
  };

  const miscClasses = {
    'button--large': size === 'large',
    'button--small': size === 'small',
    'button--active': active,
    'button--compact': compact,
  };

  const externalClasses = {
    'klara-ui-tooltip__hoverable': tooltip,
  };

  return {
    ...primaryClasses,
    ...iconClasses,
    ...typeClasses,
    ...roundClasses,
    ...miscClasses,
    ...externalClasses,
  };
}

class Button extends Component {
  constructor(props) {
    super(props);

    this.state = {
      mouseOver: false,
    };
  }

  onMouseEnter() {
    this.setState({ mouseOver: true });
  }

  onMouseLeave() {
    this.setState({ mouseOver: false });
  }

  ref = React.createRef();

  focus() {
    if (this.ref.current) {
      this.ref.current.focus();
    }
  }

  render() {
    const {
      children,
      className,
      disabled,
      href,
      icon,
      inputProps,
      loading,
      onClick,
      rel,
      rightAlignedIcon,
      target,
      tooltip,
    } = this.props;

    const classes = classNames('button', className, resolveClasses(this.props));
    const tooltipElement = tooltip ? <Tooltip>{tooltip}</Tooltip> : null;
    const spinnerComponent = loading && (
      <Spinner
        type="fading"
        size={24}
        light={this.state.mouseOver && !disabled}
      />
    );
    const iconComponent = icon && <Icon shape={icon} />;
    const buttonIcon = spinnerComponent || iconComponent;
    const Tag = href && !disabled ? 'a' : 'button';

    return (
      <Tag
        ref={this.ref}
        className={classes}
        onMouseEnter={() => {
          this.onMouseEnter();
        }}
        onMouseLeave={() => {
          this.onMouseLeave();
        }}
        disabled={disabled}
        onClick={onClick}
        {...(href ? { href, rel, target } : {})}
        {...inputProps}
      >
        {tooltipElement}
        {!rightAlignedIcon && buttonIcon}
        {children && <span>{children}</span>}
        {rightAlignedIcon && buttonIcon}
      </Tag>
    );
  }
}

Button.defaultProps = {
  children: null,
  className: '',
  disabled: false,
  href: null,
  icon: '',
  inputProps: null,
  loading: false,
  onClick: () => {},
  primary: false,
  rel: null,
  rightAlignedIcon: false,
  roundLeft: true,
  roundRight: true,
  size: 'medium',
  target: null,
  tooltip: null,
  type: 'default',
};

Button.propTypes = {
  children: PropTypes.node,
  className: PropTypes.string,
  disabled: PropTypes.bool,
  href: PropTypes.string,
  icon: PropTypes.string,
  inputProps: PropTypes.object, //eslint-disable-line
  loading: PropTypes.bool,
  onClick: PropTypes.func,
  primary: PropTypes.bool,
  rel: PropTypes.string,
  rightAlignedIcon: PropTypes.bool,
  roundLeft: PropTypes.bool,
  roundRight: PropTypes.bool,
  size: PropTypes.oneOf(['small', 'medium', 'large']),
  target: PropTypes.string,
  tooltip: PropTypes.string,
  type: PropTypes.string,
};

Button.propDescriptions = {
  icon: 'Any valid icon name',
  href: 'Render the button as an anchor tag, pointing to the given URL',
  tooltip: "Renders with klara-ui's tooltip component. Always at left side.",
  inputProps: 'Passes props to underlying tag element',
  size: 'Can be either small, medium or large, defaults to medium',
};

export default Button;
