import React, { Component } from 'react';
import PropTypes from 'prop-types';
import moment from 'moment';
import Dategrid from 'react-dategrid';
import classNames from 'classnames';

import Timepicker from './timepicker';

import {
  LEFT_ARROW,
  RIGHT_ARROW,
  DOWN_ARROW,
  UP_ARROW,
  KEY_ENTER,
  KEY_ESCAPE,
} from '../../utils/keyConstants';

moment.locale('sv');

function modifyDateWithArrowKey({ keyCode, shiftKey, altKey }, dateToModify) {
  const date = dateToModify.clone();

  /* eslint-disable no-multi-spaces, default-case */
  switch (keyCode) {
    case shiftKey && altKey && RIGHT_ARROW:
    case shiftKey && RIGHT_ARROW:
      return date.add(1, 'month');
    case altKey && RIGHT_ARROW:
    case RIGHT_ARROW:
      return date.add(1, 'day');

    case shiftKey && altKey && LEFT_ARROW:
    case shiftKey && LEFT_ARROW:
      return date.subtract(1, 'month');
    case altKey && LEFT_ARROW:
    case LEFT_ARROW:
      return date.subtract(1, 'day');

    case shiftKey && altKey && DOWN_ARROW:
      return date.subtract(1, 'minute');
    case shiftKey && DOWN_ARROW:
      return date.subtract(1, 'hour').minutes(0);
    case altKey && DOWN_ARROW:
      return date.subtract(date.minutes() % 15 || 15, 'minutes');
    case DOWN_ARROW:
      return date.add(1, 'week');

    case shiftKey && altKey && UP_ARROW:
      return date.add(1, 'minute');
    case shiftKey && UP_ARROW:
      return date.add(1, 'hour').minutes(0);
    case altKey && UP_ARROW:
      return date.add(15 - (date.minutes() % 15), 'minutes');
    case UP_ARROW:
      return date.subtract(1, 'week');
  }
  /* eslint-enable no-multi-spaces, default-case */

  return null;
}

class Datepicker extends Component {
  state = {
    datePickerHasFocus: false,
    view: this.props.selectedDate || moment(),
  };

  componentDidUpdate(prevProps) {
    if (
      this.props.selectedDate &&
      prevProps.selectedDate !== this.props.selectedDate
    ) {
      this.setState({ view: this.props.selectedDate });
    }
  }

  onKeyUp = event => {
    switch (event.keyCode) {
      case KEY_ENTER:
        return this.props.onEnter();
      case KEY_ESCAPE:
        return this.props.onEscape();
      default:
        return null;
    }
  };

  onKeyDown = event => {
    if (
      this.state.datePickerHasFocus &&
      [LEFT_ARROW, DOWN_ARROW, UP_ARROW, RIGHT_ARROW].includes(event.keyCode)
    ) {
      event.preventDefault();
      const newDate = modifyDateWithArrowKey(
        event,
        this.props.selectedDate || moment()
      );

      if (this.props.isDateValid(newDate)) {
        this.setState({ view: newDate });
        this.onChange(newDate);
      }
    }
  };

  onDayClick = day => {
    this.setState({ view: day });
    this.onChange(day);
  };

  onChange = day => {
    this.props.onChange(day);
  };

  onViewChange = view => {
    this.setState({ view });
  };

  setDatePickerRef = el => {
    this.datePickerRef = el;
  };

  focus = () => {
    this.containerRef.focus();
  };

  containerRefCallback = el => {
    this.containerRef = el;
  };

  focusDatePicker = e => {
    if (!e.nativeEvent.path || this.containerRef === e.nativeEvent.path[0]) {
      this.setState({ datePickerHasFocus: true });
    }
  };

  blurDatePicker = () => {
    this.setState({ datePickerHasFocus: false });
  };

  render() {
    const { view } = this.state;
    const { withoutTimepicker, isDateValid, selectedDate } = this.props;

    return (
      <div
        className="kui-datepicker kui-datepicker__wrapper"
        onKeyDown={this.onKeyDown}
        onKeyUp={this.onKeyUp}
        tabIndex={0}
        onFocus={this.focusDatePicker}
        onBlur={this.blurDatePicker}
        ref={this.containerRefCallback}
      >
        <Dategrid
          className="kui-datepicker__calendar"
          view={view}
          onViewChange={this.onViewChange}
          renderTitle={currentView => {
            let str = currentView.format('MMMM YYYY');

            str = `${str.substr(0, 1).toUpperCase()}${str.substr(1)}`;

            return <div className="kui-datepicker__title">{str}</div>;
          }}
          renderPreviousNavigation={props => (
            <div className="kui-datepicker__previous" {...props}>
              ‹
            </div>
          )}
          renderNextNavigation={props => (
            <div className="kui-datepicker__next" {...props}>
              ›
            </div>
          )}
          renderWeekday={index => (
            <strong>{['Må', 'Ti', 'On', 'To', 'Fr', 'Lö', 'Sö'][index]}</strong>
          )}
          renderDay={(day, currentView) => {
            const { classNameForDate, highlightToday } = this.props;
            const classes = classNames(
              'kui-datepicker__day',
              {
                'kui-datepicker__day--today':
                  highlightToday && day.isSame(moment(), 'day'),
                'kui-datepicker__day--outside-month': !day.isSame(
                  currentView,
                  'month'
                ),
                'kui-datepicker__day--selected':
                  selectedDate && day.isSame(selectedDate, 'day'),
                'kui-datepicker__day--invalid': !isDateValid(day),
              },
              classNameForDate(day)
            );

            return (
              <div
                onClick={
                  isDateValid(day) ? () => this.onDayClick(day) : () => {}
                }
                className={classes}
              >
                {day.date()}
              </div>
            );
          }}
        />
        {!withoutTimepicker && (
          <Timepicker date={selectedDate} onChange={this.onChange} />
        )}
      </div>
    );
  }
}

Datepicker.propTypes = {
  isDateValid: PropTypes.func,
  onChange: PropTypes.func,
  highlightToday: PropTypes.bool,
  onEnter: PropTypes.func,
  onEscape: PropTypes.func,
  classNameForDate: PropTypes.func,
  selectedDate: PropTypes.instanceOf(moment),
  withoutTimepicker: PropTypes.bool,
};

Datepicker.defaultProps = {
  isDateValid: () => true,
  highlightToday: true,
  onChange: () => {},
  onEnter: () => {},
  onEscape: () => {},
  classNameForDate: () => {},
  selectedDate: null,
  withoutTimepicker: false,
};

Datepicker.propDescriptions = {
  selectedDate:
    'The currently selected date. Should be a valid Moment instance',
  onChange:
    'Function which will receive the new timestamp as a moment instance',
  withoutTimepicker: 'Should the timepicker be included?',
  disabled: 'Should the datepicker be disabled?',
  isDateValid: 'Dates in the passed should be diabled',
  classNameForDate:
    'Optionally override the classes for a given day, return string',
};

export default Datepicker;
