import React from 'react';
import { expect } from 'chai';
import { mount } from 'enzyme';
import sinon from 'sinon';
import moment from 'moment';
import Datepicker from '../datepicker';
import Timepicker from '../timepicker';
import { RIGHT_ARROW } from '../../../utils/keyConstants';

const textFind = text => el => el.text() === text;

describe('Datepicker', () => {
  let clock;
  let onChangeStub;

  beforeEach(() => {
    onChangeStub = sinon.stub();

    const now = 1516627620000; // 2018-01-22 14:27.00 UTC+1
    clock = sinon.useFakeTimers({ now });
  });

  afterEach(() => {
    clock.restore();
  });

  describe('Rendering', () => {
    it('should render with a table', () => {
      const wrapper = mount(<Datepicker />);

      expect(wrapper.find('table')).to.exist();
    });

    it('should render the currently selected month above the day grid', () => {
      const wrapper = mount(<Datepicker />);

      expect(
        wrapper
          .find('tr')
          .first()
          .find('th')
          .at(1)
          .text()
      ).to.contain('Januari 2018');
    });

    it('should show the weekdays', () => {
      const wrapper = mount(<Datepicker />);

      expect(
        wrapper
          .find('tr')
          .at(1)
          .text()
      ).to.equal('MåTiOnToFrLöSö');
    });
  });

  it('should highlight the given date when passing the getSelectedProp prop', () => {
    const wrapper = mount(
      <Datepicker selectedDate={moment().add(2, 'week')} />
    );

    expect(wrapper.find('.kui-datepicker__day--selected').text()).to.equal('5');
  });

  it('should display the given time when passing the getSelectedProp prop', () => {
    const wrapper = mount(
      <Datepicker
        selectedDate={moment()
          .add(3, 'hour')
          .subtract(13, 'minutes')}
      />
    );

    expect(wrapper.find('input').prop('value')).to.contain('17:14');
  });

  it('should call the given onChange handler as the date changes', () => {
    const wrapper = mount(<Datepicker onChange={onChangeStub} />);

    wrapper.find('.kui-datepicker__next').simulate('click');
    wrapper
      .find('.kui-datepicker__day')
      .filterWhere(textFind('12'))
      .simulate('click');

    const expectedMomentInstance = moment()
      .add(1, 'month')
      .date(12)
      .toString();
    const actualMomentInstance = onChangeStub.getCall(0).args[0].toString();

    expect(actualMomentInstance).to.equal(expectedMomentInstance);
  });

  it('should change date while pressing arrow buttons', () => {
    const wrapper = mount(<Datepicker onChange={onChangeStub} />);

    wrapper.simulate('focus');
    wrapper.simulate('keydown', { keyCode: RIGHT_ARROW });

    const expectedMomentInstance = moment()
      .add(1, 'day')
      .toString();
    const actualMomentInstance = onChangeStub.getCall(0).args[0].toString();

    expect(expectedMomentInstance).to.equal(actualMomentInstance);
  });

  describe('Timepicker', () => {
    it('should default to "--:--" if no date is specified', () => {
      const wrapper = mount(<Datepicker />);

      expect(wrapper.find('input').prop('value')).to.contain('--:--');
    });

    it('should render a timepicker depending on a prop', () => {
      expect(mount(<Datepicker />).find(Timepicker)).to.exist();
      expect(
        mount(<Datepicker withoutTimepicker />).find(Timepicker)
      ).to.not.exist();
    });

    it('should produce a moment instance while changing time', () => {
      const timepicker = mount(<Datepicker onChange={onChangeStub} />).find(
        Timepicker
      );

      const newHour = '15';
      const newMinute = '27';

      timepicker
        .find('input')
        .simulate('change', { target: { value: `${newHour}:${newMinute}` } });

      expect(onChangeStub.getCall(0).args[0].toString()).to.equal(
        moment()
          .hour(newHour)
          .minute(newMinute)
          .toString()
      );
    });
  });
});
