import React from 'react';
import classNames from 'classnames';
import PropTypes from 'prop-types';

import Button from '../button/button';
import DropdownOption from './DropdownOption';
import Badge from '../badge/badge';
import AnchoredMenu from './AnchoredMenu';

const block = 'drop-down';

class Dropdown extends React.Component {
  state = {
    highlightedIndex: -1,
  };

  isSelected = value =>
    [].concat(this.props.selectedValue).indexOf(value) !== -1;

  handleMouseEnterOption = hoveredOption => {
    const { options } = this.props;
    this.setState({
      highlightedIndex: options.findIndex(
        ({ value }) => value === hoveredOption
      ),
    });
  };

  handleMouseLeaveOption = () => {
    this.setState({ highlightedIndex: -1 });
  };

  handleClose = () => {
    if (!this.props.multiple) {
      this.setState({
        highlightedIndex: this.props.options.findIndex(
          ({ value }) => value === this.props.selectedValue
        ),
      });
    }
  };

  handleArrowKey = arrowKey => {
    const { options } = this.props;
    const direction = arrowKey === 'ArrowUp' ? -1 : 1;
    const newHighlightedIndex =
      (options.length +
        // Using max(-1, newIndex) to cover for case with arrowUp from initial index -1 state
        Math.max(-1, this.state.highlightedIndex + direction)) %
      options.length;
    this.setState({ highlightedIndex: newHighlightedIndex });
  };

  handleValueToggled = toggledValue => {
    const value = this.props.selectedValue || [];
    const toggledValueIndex = value.indexOf(toggledValue);
    this.props.onChange(
      toggledValueIndex === -1
        ? [...value, toggledValue]
        : [
            ...value.slice(0, toggledValueIndex),
            ...value.slice(toggledValueIndex + 1),
          ]
    );
  };

  handleSelection = (value, { closeMenu, focusAnchor }) => {
    const { multiple, onChange } = this.props;
    focusAnchor();
    if (multiple) {
      this.handleValueToggled(value);
    } else {
      onChange(value);
      closeMenu();
    }
  };

  handleEnter = (event, actions) => {
    const highlightedOption = this.props.options[this.state.highlightedIndex];
    if (highlightedOption) {
      event.preventDefault();
      this.handleSelection(highlightedOption.value, actions);
    }
  };

  resolveLabel() {
    const { selectedValue, placeholder, options, multiple, type } = this.props;

    if (selectedValue === null || (multiple && selectedValue.length === 0)) {
      return type === 'ghost' ? ' ' : placeholder;
    }

    if (multiple) {
      return (
        <React.Fragment>
          {selectedValue.map(v => {
            const option = options.find(o => o.value === v);
            return (
              <Badge
                key={option.label + option.value}
                className={`${block}__badge`}
                icon="remove"
                onIconClick={event => {
                  event.stopPropagation();
                  this.handleValueToggled(option.value);
                }}
                rightAlignedIcon
              >
                {option.label}
              </Badge>
            );
          })}
        </React.Fragment>
      );
    }

    const selectedOption = options.find(
      option => option.value === selectedValue
    );
    return selectedOption ? selectedOption.label : placeholder;
  }

  renderButton = (onClick, ref, isOpen) => {
    const { placeholder, selectedValue, multiple, type } = this.props;
    const selected = multiple
      ? selectedValue && selectedValue.length > 0
      : selectedValue !== null;
    return (
      <React.Fragment>
        <Button
          className={`${block}__button ${type}`}
          icon={
            isOpen || this.props.showOptionsAbove
              ? 'chevron_up'
              : 'chevron_down'
          }
          disabled={this.props.disabled}
          onClick={onClick}
          rightAlignedIcon
          ref={ref}
          type={this.props.type}
        >
          {this.resolveLabel()}
        </Button>
        {type === 'ghost' &&
          placeholder && (
            <label className={`drop-down__label ${selected && 'selected'}`}>
              {placeholder}
            </label>
          )}
      </React.Fragment>
    );
  };

  renderOption = (option, highlighted, onClick, onMouseEnter) => (
    <DropdownOption
      key={option.value}
      value={option.value}
      label={option.label}
      isSelected={this.isSelected(option.value)}
      isHighlighted={highlighted}
      onSelect={onClick}
      onMouseEnter={onMouseEnter}
    />
  );

  render() {
    const { highlightedIndex } = this.state;
    const {
      options,
      showOptionsAbove,
      className,
      renderButton,
      renderOption,
      popoverProps,
      type,
      placeholder,
    } = this.props;

    return (
      <AnchoredMenu
        renderAnchor={renderButton || this.renderButton}
        renderMenu={actions =>
          options.map((option, index) =>
            (renderOption || this.renderOption)(
              option,
              highlightedIndex === index,
              () => this.handleSelection(option.value, actions),
              () => this.handleMouseEnterOption(option.value)
            )
          )
        }
        onArrowKey={this.handleArrowKey}
        onClose={this.handleClose}
        onEnter={this.handleEnter}
        popoverProps={{
          contentClassName: `${block}__popover`,
          withoutArrow: true,
          down: !showOptionsAbove,
          dynamicAnchorNode: true,
          ...popoverProps,
        }}
        className={classNames(
          block,
          className,
          type === 'ghost' && placeholder && 'margin'
        )}
        onMouseLeave={this.handleMouseLeaveOption}
      />
    );
  }
}
Dropdown.defaultProps = {
  className: '',
  multiple: false,
  onChange: () => {},
  renderButton: null,
  renderOption: null,
  popoverProps: {},
  options: [],
  placeholder: '',
  disabled: false,
  selectedValue: null,
  showOptionsAbove: false,
  type: 'default',
};

Dropdown.propTypes = {
  className: PropTypes.string,
  multiple: PropTypes.bool,
  onChange: PropTypes.func,
  renderButton: PropTypes.func,
  renderOption: PropTypes.func,
  disabled: PropTypes.bool,
  options: PropTypes.arrayOf(
    PropTypes.shape({
      label: PropTypes.string.isRequired,
      value: PropTypes.oneOfType([
        PropTypes.number,
        PropTypes.bool,
        PropTypes.string,
      ]),
    })
  ),
  placeholder: PropTypes.string,
  selectedValue: PropTypes.oneOfType([
    PropTypes.arrayOf(
      PropTypes.oneOfType([PropTypes.bool, PropTypes.number, PropTypes.string])
    ),
    PropTypes.bool,
    PropTypes.number,
    PropTypes.string,
  ]),
  showOptionsAbove: PropTypes.bool,
  type: PropTypes.oneOf(['default', 'ghost']),
  popoverProps: PropTypes.shape(),
};

export default Dropdown;
