import React from 'react';
import { expect } from 'chai';
import { mount } from 'enzyme';
import sinon from 'sinon';
import Button from '../../button/button';

import {
  KEY_DOWN,
  KEY_UP,
  KEY_ENTER,
  KEY_ESCAPE,
  KEY_SPACE,
} from './../../../utils/keyConstants';

import Dropdown from '../dropdown';
import DropdownOption from '../DropdownOption';
import Popover from '../../popover/popover';
import Badge from '../../badge/badge';
import Icon from '../../icon/icon';

const options = [
  { value: 1, label: 'One' },
  { value: 2, label: 'Two' },
  { value: 3, label: 'Three' },
];

const textFind = text => node => node.text() === text;

const getOption = (dropdown, label) =>
  dropdown.find(DropdownOption).filterWhere(textFind(label));

describe('Dropdown', () => {
  it('should render a Dropdown', () => {
    const dropdown = mount(
      <Dropdown placeholder="This is our placeholdertext" />
    );
    expect(dropdown).to.have.text('This is our placeholdertext');
    expect(dropdown).to.have.className('drop-down');
  });

  it('should have a set type', () => {
    const dropdown = mount(<Dropdown type="ghost" />);
    expect(dropdown.props().type).to.equal('ghost');
  });

  it('should use the supplied className', () => {
    const dropdown = mount(<Dropdown className="one__two--three" />);
    expect(dropdown).to.have.className('one__two--three');
  });

  it('should not show options when closed', () => {
    const dropdown = mount(<Dropdown options={options} />);
    expect(dropdown.find('ul')).not.to.be.present();
  });

  it('should show/hide options on click', () => {
    const dropdown = mount(<Dropdown options={options} />);
    dropdown.find(Button).simulate('click');
    expect(dropdown.find(DropdownOption)).to.exist();
    dropdown.find(Button).simulate('click');
    expect(dropdown.find(DropdownOption)).to.not.exist();
  });

  it('setting selectedValue prop should select the correct option', () => {
    const selected = options[1];
    const dropdown = mount(
      <Dropdown options={options} selectedValue={selected.value} />
    );
    expect(dropdown).to.have.text(selected.label);
    dropdown.find(Button).simulate('click');
    expect(
      dropdown
        .find(DropdownOption)
        .at(1)
        .prop('isSelected')
    ).to.equal(true);
  });

  it('should call the onchange callback with the selected value even if it is falsy', () => {
    const onChangeStub = sinon.stub();
    const ownOptions = [
      { value: 1, label: 'One' },
      { value: 0, label: 'Zero' },
      { value: false, label: 'False' },
      { value: '', label: 'Empty string' },
    ];
    const dropdown = mount(
      <Dropdown
        options={ownOptions}
        selectedValue={0}
        onChange={onChangeStub}
      />
    );
    expect(dropdown).to.have.text('Zero');

    dropdown.find(Button).simulate('click');

    getOption(dropdown, 'False').simulate('click');

    expect(onChangeStub).to.have.been.calledWith(false);
  });

  it('should open and select with arrow/enter key', () => {
    const onChange = sinon.stub();
    const dropdown = mount(<Dropdown options={options} onChange={onChange} />);

    dropdown.simulate('focus');
    expect(dropdown.find(DropdownOption)).not.to.be.present();

    dropdown.simulate('keydown', { key: 'ArrowDown', keyCode: KEY_DOWN });

    expect(dropdown.find(DropdownOption)).to.be.present();
    dropdown.simulate('keydown', { key: 'ArrowDown', keyCode: KEY_DOWN });

    dropdown.simulate('keydown', { keyCode: KEY_ENTER, key: 'Enter' });

    expect(onChange).to.have.been.calledOnce();
    expect(onChange).to.have.been.calledWith(options[0].value);
  });

  it('should open and select with space key', () => {
    const onChange = sinon.stub();
    const dropdown = mount(<Dropdown options={options} onChange={onChange} />);

    dropdown.simulate('focus');
    expect(dropdown.find(DropdownOption)).not.to.be.present();
    dropdown.simulate('keydown', { keyCode: KEY_DOWN, key: 'ArrowDown' });
    dropdown.simulate('keydown', { keyCode: KEY_DOWN, key: 'ArrowDown' });
    expect(dropdown.find(DropdownOption)).to.be.present();
    dropdown.simulate('keydown', { keyCode: KEY_SPACE, key: ' ' });

    expect(onChange).to.have.been.calledOnce();
    expect(onChange).to.have.been.calledWith(options[0].value);
  });

  it('should highlight option with mouse', () => {
    const dropdown = mount(<Dropdown options={options} />);
    dropdown.find(Button).simulate('click');

    dropdown
      .find(DropdownOption)
      .first()
      .simulate('mouseEnter');
    expect(
      dropdown
        .find(DropdownOption)
        .first()
        .find('div')
        .hasClass('drop-down-option--highlighted')
    ).to.equal(true);

    dropdown
      .find(DropdownOption)
      .last()
      .simulate('mouseEnter');
    expect(dropdown.find(DropdownOption).last()).to.have.className(
      'drop-down-option--highlighted'
    );

    dropdown.simulate('mouseLeave');
    expect(dropdown.find(DropdownOption).last()).not.to.have.className(
      'drop-down-option--highlighted'
    );
  });

  it('should highlight option with keyboard', () => {
    const dropdown = mount(<Dropdown options={options} />);
    dropdown.find(Button).simulate('click');
    dropdown.simulate('keydown', { keyCode: KEY_UP, key: 'ArrowUp' });
    expect(dropdown.find(DropdownOption).last()).to.have.className(
      'drop-down-option--highlighted'
    );
    dropdown.simulate('keydown', { keyCode: KEY_DOWN, key: 'ArrowDown' });
    expect(dropdown.find(DropdownOption).first()).to.have.className(
      'drop-down-option--highlighted'
    );
  });

  it('should close on esc', () => {
    const dropdown = mount(<Dropdown options={options} />);
    dropdown.simulate('click');
    dropdown.simulate('keydown', { keyCode: KEY_ESCAPE });
    expect(dropdown.find('ul')).not.to.be.present();
  });

  it('should show options above', () => {
    const dropdown = mount(<Dropdown options={options} showOptionsAbove />);
    dropdown.find(Button).simulate('click');
    expect(dropdown.find(Popover).prop('down')).to.equal(false);
  });

  it('should not crash if a non existent selectedValue is used', () => {
    const dropdown = mount(
      <Dropdown options={options} selectedValue="a-random-value" />
    );
    expect(dropdown).to.be.present();
  });

  describe('Multiselect', () => {
    it('should show multiple selected values ', () => {
      const dropdown = mount(
        <Dropdown multiple options={options} selectedValue={[1, 2]} />
      );

      const badgeTexts = dropdown.find(Badge).map(b => b.text());

      expect(badgeTexts).to.have.members(['One', 'Two']);
    });

    it('should filter out the correct value when clicking the icon on one of the badges', () => {
      const onChangeStub = sinon.stub();
      const dropdown = mount(
        <Dropdown
          multiple
          onChange={onChangeStub}
          options={options}
          selectedValue={[1, 2]}
        />
      );

      dropdown
        .find(Badge)
        .filterWhere(badge => badge.text() === 'One')
        .find(Icon)
        .simulate('click');

      expect(onChangeStub).to.have.been.calledWith([2]);
    });
  });
});
