import React, { useRef } from 'react';
import { findDOMNode } from 'react-dom';
import styled from 'styled-components';
import PropTypes from 'prop-types';
import Button from '../button/button';

const Wrapper = styled.div`
  &:focus {
    outline: none;
  }
`;

const ThumbnailWrapper = styled.div`
  display: flex;
`;

const Thumbnails = styled.div`
  display: -webkit-box;
  flex: 1;
  overflow: hidden;
  overflow-x: auto;
`;

const MediaSlider = ({
  items,
  activeIndex,
  onActiveIndexChange,
  renderPreview,
  renderThumbnail,
}) => {
  const selectedItem = useRef(null);

  const move = direction => {
    const newActiveIndex =
      (activeIndex + direction + items.length) % items.length;

    onActiveIndexChange(newActiveIndex);

    setTimeout(() => {
      if (selectedItem.current) {
        // eslint-disable-next-line react/no-find-dom-node
        findDOMNode(selectedItem.current).scrollIntoViewIfNeeded(false);
      }
    });
  };

  const goRight = () => move(1);

  const goLeft = () => move(-1);

  const handleNavigation = e => {
    switch (e.key) {
      case 'ArrowRight':
        e.preventDefault();
        goRight();
        break;
      case 'ArrowLeft':
        e.preventDefault();
        goLeft();
        break;
      default:
        break;
    }
  };

  return (
    <Wrapper onKeyDown={handleNavigation} tabIndex={-1}>
      {renderPreview(items[activeIndex])}

      <ThumbnailWrapper>
        {items.length > 1 && (
          <Button
            className="go-left"
            type="ghost"
            onClick={goLeft}
            icon="chevron_left"
          />
        )}
        <Thumbnails>
          {items.map((thumb, index) =>
            renderThumbnail(thumb, {
              isActive: activeIndex === index,
              ref: activeIndex === index ? selectedItem : undefined,
              index,
              onClick: () => move(index - activeIndex),
            })
          )}
        </Thumbnails>
        {items.length > 1 && (
          <Button
            className="go-right"
            type="ghost"
            onClick={goRight}
            icon="chevron_right"
          />
        )}
      </ThumbnailWrapper>
    </Wrapper>
  );
};

MediaSlider.propTypes = {
  items: PropTypes.arrayOf(
    PropTypes.shape({
      id: PropTypes.string.isRequired,
    })
  ).isRequired,
  activeIndex: PropTypes.number.isRequired,
  onActiveIndexChange: PropTypes.func.isRequired,
  renderPreview: PropTypes.func.isRequired,
  renderThumbnail: PropTypes.func.isRequired,
};

MediaSlider.defaultProps = {};

MediaSlider.propDescriptions = {
  items: 'Array of objects.',
  onActiveIndexChange: 'A function that expects an index as argument.',
  renderPreview: `media => <React.Element/>`,
  renderThumbnail: `(media, thumbnailProps) => <div {...thumbnailProps}><img src={media.src} /></div>`,
  onFocus: 'Is called when component is natively focused.',
  onBlur: 'Is called when component is natively blurred.',
};

export default MediaSlider;
