import React from 'react';
import sinon from 'sinon';
import { expect } from 'chai';
import { mount } from 'enzyme';

import MediaSlider from '../mediaSlider';

function mountMediaSlider(overrideProps) {
  const mediaArray = [
    {
      id: 'first-image',
      imageSrc: 'https://placebear.com/432/243',
      thumbSrc: 'https://placebear.com/432/243',
      youtube: 'https://www.youtube.com/embed/2HH4CiAjjvM',
    },
    {
      id: 'second-image',
      imageSrc: 'https://placebear.com/100/234',
      thumbSrc: 'https://placebear.com/100/234',
    },
    {
      id: 'third-image',
      imageSrc: 'https://placebear.com/400/225',
      thumbSrc: 'https://placebear.com/400/225',
    },
  ];

  const defaultProps = {
    items: mediaArray,
    activeIndex: 0,
    updateActiveIndex: () => {},
    renderPreview: () => <div className="preview" />,
    renderThumbnail: (thumb, thumbnailProps) => (
      <div className="thumb" {...thumbnailProps} />
    ),
  };

  return mount(<MediaSlider {...defaultProps} {...overrideProps} />);
}

describe('Media slider', () => {
  it('should render with one item in it', () => {
    const mediaSlider = mountMediaSlider();

    expect(mediaSlider.find('.preview').length).to.equal(1);
  });

  it('should render with three thumbnails in it', () => {
    const mediaSlider = mountMediaSlider();

    expect(mediaSlider.find('.thumb').length).to.equal(3);
  });

  it('should render with a image for each slide', () => {
    const mediaSlider = mountMediaSlider({
      renderPreview: media => <img src={media.imageSrc} />,
    });

    expect(mediaSlider.find('img').length).to.equal(1);
  });

  it('should render with chevron on each side', () => {
    const mediaSlider = mountMediaSlider({
      activeIndex: 1,
    });

    expect(mediaSlider.find('.go-left button').length).to.equal(1);
    expect(mediaSlider.find('.go-right button').length).to.equal(1);
  });

  it('should slide to next slide when right chevron is clicked', () => {
    const updateActiveIndex = sinon.spy();

    const mediaSlider = mountMediaSlider({
      activeIndex: 1,
      onActiveIndexChange: updateActiveIndex,
    });

    mediaSlider.find('.go-right button').simulate('click');

    expect(updateActiveIndex).to.have.been.called.with(2);
  });

  it('should slide to previous slide when left chevron is clicked', () => {
    const updateActiveIndex = sinon.spy();

    const mediaSlider = mountMediaSlider({
      activeIndex: 2,
      onActiveIndexChange: updateActiveIndex,
    });

    mediaSlider.find('.go-left button').simulate('click');

    expect(updateActiveIndex).to.have.been.called.with(1);
  });

  it('should slide to last slide when last thumbnail is clicked', () => {
    const updateActiveIndex = sinon.spy();

    const mediaSlider = mountMediaSlider({
      onActiveIndexChange: updateActiveIndex,
    });

    mediaSlider
      .find('.thumb')
      .last()
      .simulate('click');

    expect(updateActiveIndex).to.have.been.called.with(2);
  });
});
