import React from 'react';
import classNames from 'classnames';
import PropTypes from 'prop-types';
import Notification from '../notification/notification';

const componentClassName = 'klara-ui-modal';

class Modal extends React.Component {
  componentDidMount() {
    window.addEventListener('keyup', this.onKeyUp);
  }

  componentWillUnmount() {
    window.removeEventListener('keyup', this.onKeyUp);
  }

  onContainerClick = ({ target }) => {
    if (target === this.containerRef.current) {
      this.props.onClose();
    }
  };

  onKeyUp = ({ key }) => {
    if (key === 'Escape') {
      this.props.onClose();
    }
  };

  containerRef = React.createRef();

  render() {
    const {
      children,
      title,
      notification,
      renderHeader,
      renderFooter,
      onDismissNotification,
      className,
    } = this.props;

    return (
      <div
        className={classNames(componentClassName, className)}
        onClick={this.onContainerClick}
        ref={this.containerRef}
      >
        <div className={`${componentClassName}__container`}>
          <header>{renderHeader(title, `${componentClassName}__title`)}</header>

          <div className={`${componentClassName}__content`}>{children}</div>

          <div className={`${componentClassName}__notifications`}>
            {notification && (
              <Notification
                dismissable={notification.dismissable}
                onDismiss={onDismissNotification}
                title={notification.title}
                description={notification.description || ''}
                type={notification.type}
              />
            )}
          </div>
          {renderFooter && <footer>{renderFooter()}</footer>}
        </div>
      </div>
    );
  }
}

Modal.propTypes = {
  children: PropTypes.oneOfType([
    PropTypes.arrayOf(PropTypes.node),
    PropTypes.node,
  ]).isRequired,
  title: PropTypes.string.isRequired,
  className: PropTypes.string,
  renderHeader: PropTypes.func,
  notification: PropTypes.shape({
    dismissable: PropTypes.bool,
    title: PropTypes.string,
    description: PropTypes.description,
    type: PropTypes.string,
  }),
  onDismissNotification: PropTypes.func,
  onClose: PropTypes.func,
  renderFooter: PropTypes.func,
};

Modal.defaultProps = {
  notification: null,
  renderHeader: (title, suggestedClassName) => (
    <h2 className={suggestedClassName}>{title}</h2>
  ),
  onClose: () => {},
  onDismissNotification: () => {},
  className: '',
  renderFooter: null,
};

Modal.propDescriptions = {
  notification:
    'An object with the properties dismissable, title, description and type',
  onClose: 'Fires on pressing Escape, or when clicking outside the modal',
  renderFooter: 'A function that returns JSX',
};

export default Modal;
