import React from 'react';
import { findDOMNode } from 'react-dom';
import PropTypes from 'prop-types';
import OutsideClickHandler from 'react-outside-click-handler';

const TRIANGLE_HEIGHT = 6;
const TRIANGLE_WIDTH = 12;

export default class Popover extends React.Component {
  static propTypes = {
    withoutArrow: PropTypes.bool,
    anchorPoint: PropTypes.shape({
      left: PropTypes.number.isRequired,
      top: PropTypes.number.isRequired,
    }),
    anchorNode: PropTypes.oneOfType([
      PropTypes.instanceOf(HTMLElement),
      PropTypes.instanceOf(React.Component),
    ]),
    children: PropTypes.node.isRequired,
    className: PropTypes.string,
    contentClassName: PropTypes.string,
    min: PropTypes.number,
    dynamicAnchorNode: PropTypes.bool,
    dynamicContent: PropTypes.bool,
    max: PropTypes.number,
    down: PropTypes.bool,
    left: PropTypes.bool,
    right: PropTypes.bool,
    onOutsideClick: PropTypes.func,
  };

  static propDescriptions = {
    onOutsideClick:
      'Provide a function that decides what happens if you click outside the popover, i.e. togglePopover',
  };

  static defaultProps = {
    withoutArrow: false,
    anchorPoint: null,
    dynamicAnchorNode: false,
    dynamicContent: false,
    className: '',
    contentClassName: '',
    down: false,
    anchorNode: null,
    left: false,
    right: false,
    min: Number.MIN_SAFE_INTEGER,
    max: Number.MAX_SAFE_INTEGER,
    onOutsideClick: null,
  };

  state = {
    contentSize: null,
    anchorPoint: this.props.anchorNode ? this.getAnchorPointFromNode() : null,
  };

  componentDidMount() {
    this.setState({
      contentSize: this.getContentSize(),
    });
    this.openedBy = document.activeElement;
  }

  // eslint-disable-next-line camelcase
  UNSAFE_componentWillReceiveProps({ dynamicAnchorNode, dynamicContent }) {
    if (dynamicAnchorNode || dynamicContent) {
      this.setState({
        needsRecalculation: true,
      });
    }
  }

  componentDidUpdate() {
    const { dynamicAnchorNode, dynamicContent } = this.props;
    const { needsRecalculation, anchorPoint, contentSize } = this.state;
    if (needsRecalculation) {
      this.setState({
        anchorPoint: dynamicAnchorNode
          ? this.getAnchorPointFromNode()
          : anchorPoint,
        contentSize: dynamicContent ? this.getContentSize() : contentSize,
        needsRecalculation: false,
      });
    }
  }

  setContentRef = ref => {
    this.contentRef = ref;
  };

  getContentSize() {
    const { left, right } = this.props;
    const { width, height } = this.contentRef.getBoundingClientRect();
    return left || right ? height : width;
  }

  getPositionForAxis(node) {
    const { min, max, left, right } = this.props;
    const anchorStart = left || right ? node.offsetTop : node.offsetLeft;
    const anchorSize = left || right ? node.offsetHeight : node.offsetWidth;
    return Math.min(
      max - TRIANGLE_WIDTH,
      Math.max(min + TRIANGLE_WIDTH, anchorStart + anchorSize / 2)
    );
  }

  getAnchorPointFromNode() {
    const { anchorNode, right, down, left } = this.props;
    const node = findDOMNode(anchorNode); // eslint-disable-line react/no-find-dom-node
    const axisPosition = this.getPositionForAxis(node);
    return left
      ? { top: axisPosition, left: node.offsetLeft }
      : right
      ? { top: axisPosition, left: node.offsetLeft + node.offsetWidth }
      : down
      ? { left: axisPosition, top: node.offsetTop + node.offsetHeight }
      : // upward
        { left: axisPosition, top: node.offsetTop };
  }

  handleOutsideClick = e => {
    if (!this.props.onOutsideClick) return;
    if (this.openedBy.contains(e.target)) return;

    this.props.onOutsideClick();
  };

  render() {
    const {
      children,
      className,
      contentClassName,
      min,
      max,
      down,
      left,
      right,
      withoutArrow,
    } = this.props;

    const anchorPoint = this.props.anchorPoint || this.state.anchorPoint;
    const contentSize = this.state.contentSize;

    const absoluteDirection = right
      ? 'left'
      : left
      ? 'right'
      : down
      ? 'top'
      : 'bottom';

    const contentStyle = {
      [absoluteDirection]: TRIANGLE_HEIGHT,
    };
    const positionAdjustmentProperty = left || right ? 'top' : 'left';

    if (contentSize) {
      const requiredLeadingAdjustment = Math.max(
        0,
        min + contentSize / 2 - anchorPoint[positionAdjustmentProperty]
      );
      const requiredTrailingAdjustment = Math.max(
        0,
        anchorPoint[positionAdjustmentProperty] + contentSize / 2 - max
      );
      Object.assign(contentStyle, {
        [positionAdjustmentProperty]:
          -contentSize / 2 +
          requiredLeadingAdjustment -
          requiredTrailingAdjustment,
      });
    }
    return (
      <OutsideClickHandler onOutsideClick={this.handleOutsideClick}>
        <div
          className={`klara-ui-popover ${className}`}
          style={{
            left: anchorPoint.left,
            top: anchorPoint.top,
            [left || right ? 'height' : 'width']: '100%',
          }}
        >
          <div
            ref={this.setContentRef}
            style={contentStyle}
            className={`klara-ui-popover__content ${contentClassName}`}
          >
            {children}
          </div>
          {!withoutArrow && (
            <div
              className="klara-ui-popover__attach-arrow"
              style={{
                [absoluteDirection]: 0,
                [positionAdjustmentProperty]: -TRIANGLE_HEIGHT,
                transform: `rotate(${
                  right ? 135 : left ? 315 : down ? 225 : 45
                }deg)`,
              }}
            />
          )}
        </div>
      </OutsideClickHandler>
    );
  }
}
