import React from 'react';
import PropTypes from 'prop-types';
import { TransitionGroup, CSSTransition } from 'react-transition-group';
import { DragDropContext, Droppable, Draggable } from 'react-beautiful-dnd';
import { replace } from 'lodash/fp';
import Tab from './tab';
import NewTab from './newTab';
import OverflowIndicator from './overflowIndicator';

const OVERFLOW_THRESHOLD = 20;
const SCROLL_STEP = 180;
const SCROLL_STEP_FRAME = 10;

const transformDragTransform = props => ({
  transform: replace(/, -?[\w].*px/, ', 0', props.style.transform),
});

class Tabs extends React.Component {
  state = {
    overflowingLeft: false,
    overflowingRight: false,
    scrollTarget: null,
  };

  componentDidMount() {
    this.checkOverflows();

    this.containerRef.addEventListener('scroll', this.checkOverflows);
    window.addEventListener('resize', this.checkOverflows);
  }

  componentDidUpdate(prevProps) {
    if (this.props.tabs.length === prevProps.tabs.length + 1) {
      this.containerRef.scrollLeft = this.containerRef.scrollWidth;
    }

    if (prevProps.tabs.length !== this.props.tabs.length) {
      this.checkOverflows({ delay: 250 });
    }
  }

  componentWillUnmount() {
    clearTimeout(this.overflowCheckTimeout);
    this.containerRef.removeEventListener('scroll', this.checkOverflows);
    window.removeEventListener('resize', this.checkOverflows);
  }

  onLeftOverflowIndicatorClick = () => {
    this.setState(
      { scrollTarget: this.containerRef.scrollLeft - SCROLL_STEP },
      () => {
        requestAnimationFrame(this.scrollAnimationFrame);
      }
    );
  };

  onRightOverflowIndicatorClick = () => {
    this.setState(
      { scrollTarget: this.containerRef.scrollLeft + SCROLL_STEP },
      () => {
        requestAnimationFrame(this.scrollAnimationFrame);
      }
    );
  };

  onDragEnd = result => {
    // dropped outside the list
    if (!result.destination) {
      return;
    }
    this.props.onReorder(result.source.index, result.destination.index);
  };

  scrollAnimationFrame = () => {
    const { scrollTarget } = this.state;
    const { scrollLeft: currentPosition } = this.containerRef;

    if (scrollTarget) {
      if (Math.abs(scrollTarget - currentPosition) < SCROLL_STEP_FRAME) {
        this.containerRef.scrollLeft = scrollTarget;
        this.setState({ scrollTarget: null });
      } else if (scrollTarget > currentPosition) {
        this.containerRef.scrollLeft += SCROLL_STEP_FRAME;
      } else {
        this.containerRef.scrollLeft -= SCROLL_STEP_FRAME;
      }

      requestAnimationFrame(this.scrollAnimationFrame);
    }
  };

  containerRefCallback = el => {
    this.containerRef = el;
  };

  checkOverflows = ({ delay } = { delay: 0 }) => {
    this.overflowCheckTimeout = setTimeout(() => {
      if (!this.containerRef) return;
      const { scrollLeft, scrollWidth, clientWidth } = this.containerRef;

      const overflowingLeft = scrollLeft > OVERFLOW_THRESHOLD;
      const overflowingRight =
        scrollLeft + clientWidth + OVERFLOW_THRESHOLD < scrollWidth;

      this.setState({ overflowingLeft, overflowingRight });
    }, delay);
  };

  render() {
    const { overflowingLeft, overflowingRight } = this.state;
    const {
      tabs,
      activeTab,
      onTabClick,
      onTabClose,
      onNewTabClick,
      canCloseLastTab,
    } = this.props;

    return (
      <div className="klara-ui-tabs">
        <OverflowIndicator
          direction="left"
          visible={overflowingLeft}
          onClick={this.onLeftOverflowIndicatorClick}
        />
        <OverflowIndicator
          direction="right"
          visible={overflowingRight}
          onClick={this.onRightOverflowIndicatorClick}
        />
        <div className="klara-ui-tabs__inner" ref={this.containerRefCallback}>
          <DragDropContext onDragEnd={this.onDragEnd}>
            <Droppable droppableId="droppable" direction="horizontal">
              {drop => (
                <div ref={drop.innerRef} {...drop.droppableProps}>
                  <TransitionGroup className="klara-ui-tabs__transition-group">
                    {tabs.map((tab, index) => (
                      <CSSTransition
                        key={tab.id}
                        timeout={150}
                        classNames="klara-ui-tabs__tab"
                      >
                        <Draggable draggableId={tab.id} index={index}>
                          {(drag, snapshot) => (
                            <div
                              ref={drag.innerRef}
                              {...drag.draggableProps}
                              {...drag.dragHandleProps}
                              style={{
                                ...drag.draggableProps.style,
                                ...(snapshot.isDragging
                                  ? transformDragTransform(drag.draggableProps)
                                  : {}),
                              }}
                            >
                              <Tab
                                tab={tab}
                                immortal={
                                  typeof canCloseLastTab === 'boolean'
                                    ? !canCloseLastTab
                                    : canCloseLastTab(tabs)
                                }
                                active={tab.id === activeTab}
                                onClick={onTabClick}
                                onClose={onTabClose}
                              />
                            </div>
                          )}
                        </Draggable>
                      </CSSTransition>
                    ))}
                    <NewTab onClick={onNewTabClick} />
                  </TransitionGroup>
                </div>
              )}
            </Droppable>
          </DragDropContext>
        </div>
      </div>
    );
  }
}

Tabs.NewTab = NewTab;
Tabs.Tab = Tab;

Tabs.propTypes = {
  activeTab: PropTypes.string,
  tabs: PropTypes.arrayOf(
    PropTypes.shape({
      id: PropTypes.string.isRequired,
      title: PropTypes.oneOfType([PropTypes.string, PropTypes.object]),
    })
  ),
  onTabClose: PropTypes.func,
  onTabClick: PropTypes.func,
  onNewTabClick: PropTypes.func,
  onReorder: PropTypes.func,
  canCloseLastTab: PropTypes.oneOfType([PropTypes.func, PropTypes.bool]),
};

Tabs.defaultProps = {
  activeTab: null,
  tabs: [],
  onTabClose: () => {},
  onTabClick: () => {},
  onNewTabClick: undefined,
  onReorder: () => {},
  canCloseLastTab: tabs => tabs.length === 1,
};

Tabs.propsDescriptions = {};

export default Tabs;
