import React from 'react';
import PropTypes from 'prop-types';
import classnames from 'classnames';

import Spinner from '../spinner/spinner';
import FieldCounter from './fieldCounter';

import InputText from './inputText';
import InputMultiLine from './inputMultiLine';

class TextField extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      focused: false,
    };
  }

  onWrapFocus = e => {
    const { disabled } = this.props;

    if (disabled || e.target !== this.containerRef) {
      return;
    }

    this.focus();
  };

  onFocus = e => {
    this.props.onFocus(e);
    this.setState({ focused: true });
  };

  onBlur = e => {
    this.props.onBlur(e);
    this.setState({ focused: false });
  };

  onChange = value => {
    this.props.onChange(value);
  };

  onKeyPress = event => {
    const { onEnter } = this.props;

    if (event.key === 'Enter') {
      onEnter(event);
    }

    this.props.onKeyPress(event);
  };

  onInput = event => {
    this.props.onInput(event);
  };

  getPlaceholder() {
    const { label, staticLabel, placeholder, value } = this.props;

    const { focused } = this.state;

    if (value) {
      return '';
    }

    if (label && (!focused && !staticLabel)) {
      return '';
    }

    return placeholder;
  }

  focus = () => {
    this.inputRef.focus();
  };

  clear = () => {
    this.inputRef.textContent = '';
    this.inputRef.value = '';
  };

  pickInput(inputProps) {
    const { multiLine, renderInput } = this.props;

    if (renderInput) {
      return renderInput(inputProps);
    }

    const Input = multiLine ? InputMultiLine : InputText;
    return <Input {...inputProps} />;
  }

  renderIcon() {
    const icon = this.props.loading ? (
      <Spinner type="fading" size={18} />
    ) : (
      this.props.icon
    );
    return icon && <span className="text-field__icon">{icon}</span>;
  }

  render() {
    const { focused } = this.state;

    const {
      counterFunc,
      className,
      disabled,
      errorText,
      helperText,
      label,
      markRequired,
      maxLength,
      min,
      renderHelperText,
      shouldClipAtMaxLength,
      staticLabel,
      step,
      type,
      value,
      preventFocus,
    } = this.props;

    const labelText = label + (markRequired ? ' (obligatoriskt)' : '');
    const nativeMaxLength = (shouldClipAtMaxLength && maxLength) || -1;

    const filled = staticLabel || `${value}`.length > 0;

    const inputProps = {
      className: 'text-field__control',
      disabled,
      maxLength: nativeMaxLength,
      min,
      onBlur: this.onBlur,
      onChange: this.onChange,
      onFocus: this.onFocus,
      placeholder: this.getPlaceholder(),
      onKeyPress: this.onKeyPress,
      onInput: this.onInput,
      ref: input => {
        this.inputRef = input;
      },
      step,
      type,
      value,
    };
    const input = this.pickInput(inputProps);

    const rootClasses = classnames('text-field', className, {
      'text-field--filled': filled,
      'text-field--has-label': label,
      'text-field--invalid': errorText,
      'text-field--focused': focused && !preventFocus,
      'text-field--disabled': disabled,
    });

    return (
      <div
        tabIndex="-1"
        className={rootClasses}
        onFocus={this.onWrapFocus}
        ref={ref => {
          this.containerRef = ref;
        }}
      >
        {label && <label className="text-field__label">{labelText}</label>}
        <div className="text-field__input-container">
          {input}
          {this.renderIcon()}
        </div>

        <div className="text-field__floor-wrap">
          <hr className="text-field__floor" />
        </div>

        <div className="text-field__footer">
          {errorText && (
            <div className="text-field__error-text">{errorText}</div>
          )}

          {!errorText &&
            helperText &&
            !renderHelperText && (
              <div className="text-field__helper-text">{helperText}</div>
            )}

          {!errorText &&
            renderHelperText && (
              <div className="text-field__helper-text">
                {renderHelperText()}
              </div>
            )}

          <FieldCounter
            className="text-field__counter"
            maxLength={maxLength}
            value={value}
            counterFunc={counterFunc}
          />
        </div>
      </div>
    );
  }
}

TextField.propTypes = {
  className: PropTypes.string,
  counterFunc: PropTypes.func,
  disabled: PropTypes.bool,
  errorText: PropTypes.string,
  helperText: PropTypes.string,
  label: PropTypes.string,
  loading: PropTypes.bool,
  icon: PropTypes.element,
  markRequired: PropTypes.bool,
  maxLength: PropTypes.number,
  min: PropTypes.number,
  multiLine: PropTypes.bool,
  onBlur: PropTypes.func,
  onChange: PropTypes.func,
  onKeyPress: PropTypes.func,
  onInput: PropTypes.func,
  onFocus: PropTypes.func,
  onEnter: PropTypes.func,
  placeholder: PropTypes.string,
  renderHelperText: PropTypes.func,
  renderInput: PropTypes.func,
  shouldClipAtMaxLength: PropTypes.bool,
  staticLabel: PropTypes.bool,
  step: PropTypes.number,
  type: PropTypes.string,
  value: PropTypes.oneOfType([
    PropTypes.object,
    PropTypes.string,
    PropTypes.number,
  ]).isRequired,
  preventFocus: PropTypes.bool,
};

TextField.defaultProps = {
  className: '',
  counterFunc: (max, value) => max - value.length,
  disabled: false,
  errorText: '',
  helperText: '',
  label: '',
  loading: false,
  icon: null,
  markRequired: false,
  maxLength: -1,
  min: null,
  multiLine: false,
  onBlur: () => {},
  onChange: () => {},
  onKeyPress: () => {},
  onInput: () => {},
  onFocus: () => {},
  onEnter: () => {},
  placeholder: '',
  renderHelperText: null,
  renderInput: null,
  shouldClipAtMaxLength: false,
  staticLabel: false,
  step: 1,
  type: 'text',
  preventFocus: false,
};

TextField.propDescriptions = {
  className: 'Any combination of CSS classnames',
  counterFunc:
    'Function that returns the value shown in FieldCounter. Default: (max, value) => max - value.length',
  disabled: 'Controls whether user can interact with control or not',
  errorText:
    "If set triggers error-state for control, also shown under entered text describing what's happened",
  helperText: 'One more descriptive string, displayed under entered text',
  label: 'A descriptive label.',
  loading: 'Controls whether or not to display loading spinner',
  icon: 'Optionally supply an icon that will display to the right of the text',
  markRequired:
    "Show '*' sign after label name. You still have to implement validation yourself ;)",
  maxLength: 'If set will show backward counter for current text length',
  multiLine: 'If set will use expandable textarea instead of plain input',
  onBlur: 'Callback which runs with internal input blur event',
  onChange: 'Callback which runs as the field changes value',
  onKeyPress: 'Callback which runs on keypress',
  onFocus: 'Callback which runs with internal input focus event',
  onEnter: 'Callback which runs when enter is pressed in the input',
  placeholder:
    "Additional descriptive string, will be displayed on label's place when field is active",
  shouldClipAtMaxLength:
    "If set will disallow text after max lenght limit is reached. However it won't clip value coming from props",
  value: 'String value displayed in the field',
  renderInput: 'custom renderProp for the input field',
};

export default TextField;
